@@ -23,7 +23,7 @@ from pilot.util.auxiliary import set_pilot_state, show_memory_usage
 from pilot.util.container import execute
 from pilot.util.constants import UTILITY_BEFORE_PAYLOAD, UTILITY_WITH_PAYLOAD, UTILITY_AFTER_PAYLOAD_STARTED, \
     UTILITY_AFTER_PAYLOAD_FINISHED, PILOT_PRE_SETUP, PILOT_POST_SETUP, PILOT_PRE_PAYLOAD, PILOT_POST_PAYLOAD, \
-    UTILITY_AFTER_PAYLOAD_STARTED2
+    UTILITY_AFTER_PAYLOAD_STARTED2, UTILITY_AFTER_PAYLOAD_FINISHED2
 from pilot.util.filehandling import write_file
 from pilot.util.processes import kill_processes
 from pilot.util.timing import add_to_pilot_timing
@@ -92,7 +92,7 @@ class Executor(object):
         cmd_dictionary = user.get_utility_commands(order=UTILITY_BEFORE_PAYLOAD, job=job)
         if cmd_dictionary:
             cmd = '%s %s' % (cmd_dictionary.get('command'), cmd_dictionary.get('args'))
-            logger.debug('utility command to be executed before the payload: %s' % cmd)
+            logger.info('utility command (\'%s\') to be executed before the payload: %s', cmd_dictionary.get('label', 'utility'), cmd)
 
         return cmd
 
@@ -114,7 +114,7 @@ class Executor(object):
         cmd_dictionary = user.get_utility_commands(order=UTILITY_WITH_PAYLOAD, job=job)
         if cmd_dictionary:
             cmd = '%s %s' % (cmd_dictionary.get('command'), cmd_dictionary.get('args'))
-            logger.debug('utility command to be executed with the payload: %s' % cmd)
+            logger.info('utility command (\'%s\') to be executed with the payload: %s', cmd_dictionary.get('label', 'utility'), cmd)
 
         return cmd
 
@@ -138,7 +138,7 @@ class Executor(object):
         cmd_dictionary = user.get_utility_commands(order=order, job=self.__job)
         if cmd_dictionary:
             cmd = '%s %s' % (cmd_dictionary.get('command'), cmd_dictionary.get('args'))
-            logger.info('utility command to be executed after the payload: %s' % cmd)
+            logger.info('utility command (\'%s\') to be executed after the payload: %s', cmd_dictionary.get('label', 'utility'), cmd)
 
         return cmd
 
@@ -156,7 +156,7 @@ class Executor(object):
         cmd_dictionary = user.get_utility_commands(order=UTILITY_AFTER_PAYLOAD_STARTED, job=job)
         if cmd_dictionary:
             cmd = '%s %s' % (cmd_dictionary.get('command'), cmd_dictionary.get('args'))
-            logger.info('utility command to be executed after the payload: %s' % cmd)
+            logger.info('utility command to be executed after the payload: %s', cmd)
 
             # how should this command be executed?
             utilitycommand = user.get_utility_command_setup(cmd_dictionary.get('command'), job)
@@ -166,8 +166,8 @@ class Executor(object):
             try:
                 proc1 = execute(utilitycommand, workdir=job.workdir, returnproc=True, usecontainer=False,
                                 stdout=PIPE, stderr=PIPE, cwd=job.workdir, job=job)
-            except Exception as e:
-                logger.error('could not execute: %s' % e)
+            except Exception as error:
+                logger.error('could not execute: %s', error)
             else:
                 # store process handle in job object, and keep track on how many times the command has been launched
                 # also store the full command in case it needs to be restarted later (by the job_monitor() thread)
@@ -191,7 +191,7 @@ class Executor(object):
         cmd_dictionary = user.get_utility_commands(order=UTILITY_AFTER_PAYLOAD_STARTED, job=job)
         if cmd_dictionary:
             cmd = '%s %s' % (cmd_dictionary.get('command'), cmd_dictionary.get('args'))
-            logger.info('utility command to be executed after the payload: %s' % cmd)
+            logger.info('utility command to be executed after the payload: %s', cmd)
 
         return cmd
 
@@ -203,22 +203,24 @@ class Executor(object):
 #            try:
 #                proc = execute(utilitycommand, workdir=job.workdir, returnproc=True, usecontainer=False,
 #                               stdout=PIPE, stderr=PIPE, cwd=job.workdir, job=job)
-#            except Exception as e:
-#                logger.error('could not execute: %s' % e)
+#            except Exception as error:
+#                logger.error('could not execute: %s', error)
 #            else:
 #                # store process handle in job object, and keep track on how many times the command has been launched
 #                # also store the full command in case it needs to be restarted later (by the job_monitor() thread)
 #                job.utilities[cmd_dictionary.get('command')] = [proc, 1, utilitycommand]
 
-    def utility_after_payload_finished(self, job):
+    def utility_after_payload_finished(self, job, order):
         """
         Prepare commands/utilities to run after payload has finished.
 
         This command will be executed later.
 
-        REFACTOR
+        The order constant can be UTILITY_AFTER_PAYLOAD_FINISHED, UTILITY_AFTER_PAYLOAD_FINISHED2
 
         :param job: job object.
+        :param order: constant used for utility selection (constant).
+        :return: command (string), label (string).
         """
 
         cmd = ""
@@ -228,12 +230,12 @@ class Executor(object):
         user = __import__('pilot.user.%s.common' % pilot_user, globals(), locals(), [pilot_user], 0)  # Python 2/3
 
         # should any additional commands be prepended to the payload execution string?
-        cmd_dictionary = user.get_utility_commands(order=UTILITY_AFTER_PAYLOAD_FINISHED, job=job)
+        cmd_dictionary = user.get_utility_commands(order=order, job=job)
         if cmd_dictionary:
             cmd = '%s %s' % (cmd_dictionary.get('command'), cmd_dictionary.get('args'))
-            logger.debug('utility command to be executed after the payload has finished: %s' % cmd)
+            logger.info('utility command (\'%s\') to be executed after the payload has finished: %s', cmd_dictionary.get('label', 'utility'), cmd)
 
-        return cmd
+        return cmd, cmd_dictionary.get('label')
 
     def execute_utility_command(self, cmd, job, label):
         """
@@ -247,15 +249,18 @@ class Executor(object):
 
         exit_code, stdout, stderr = execute(cmd, workdir=job.workdir, cwd=job.workdir, usecontainer=False)
         if exit_code:
-            logger.warning('command returned non-zero exit code: %s (exit code = %d) - see utility logs for details' % (cmd, exit_code))
+            ignored_exit_codes = [160, 161, 162]
+            logger.warning('command returned non-zero exit code: %s (exit code = %d) - see utility logs for details', cmd, exit_code)
             if label == 'preprocess':
                 err = errors.PREPROCESSFAILURE
             elif label == 'postprocess':
                 err = errors.POSTPROCESSFAILURE
             else:
                 err = 0  # ie ignore
-            if err and exit_code != 160:  # ignore no-more-data-points exit code
+            if err and exit_code not in ignored_exit_codes:  # ignore no-more-data-points exit codes
                 job.piloterrorcodes, job.piloterrordiags = errors.add_error_code(err)
+            if exit_code in ignored_exit_codes:
+                job.transexitcode = exit_code
 
         # write output to log files
         self.write_utility_output(job.workdir, label, stdout, stderr)
@@ -285,13 +290,20 @@ class Executor(object):
             elif step == 'postprocess':
                 self.__postprocess_stdout_name = name_stdout
                 self.__postprocess_stderr_name = name_stderr
-            write_file(os.path.join(workdir, step + '_stdout.txt'), stdout, unique=True)
-        except PilotException as e:
-            logger.warning('failed to write utility stdout to file: %s, %s' % (e, stdout))
+            name = os.path.join(workdir, step + '_stdout.txt')
+            write_file(name, stdout, unique=True)
+        except PilotException as error:
+            logger.warning('failed to write utility stdout to file: %s, %s', error, stdout)
+        else:
+            logger.debug('wrote %s', name)
+
         try:
-            write_file(os.path.join(workdir, step + '_stderr.txt'), stderr, unique=True)
-        except PilotException as e:
-            logger.warning('failed to write utility stderr to file: %s, %s' % (e, stderr))
+            name = os.path.join(workdir, step + '_stderr.txt')
+            write_file(name, stderr, unique=True)
+        except PilotException as error:
+            logger.warning('failed to write utility stderr to file: %s, %s', error, stderr)
+        else:
+            logger.debug('wrote %s', name)
 
     def pre_payload(self, job):
         """
@@ -322,13 +334,13 @@ class Executor(object):
         """
 
         if label:
-            logger.info('\n\n%s:\n\n%s\n' % (label, cmd))
+            logger.info('\n\n%s:\n\n%s\n', label, cmd)
         if label == 'coprocess':
             try:
                 out = open(os.path.join(self.__job.workdir, self.__coprocess_stdout_name), 'wb')
                 err = open(os.path.join(self.__job.workdir, self.__coprocess_stderr_name), 'wb')
-            except Exception as e:
-                logger.warning('failed to open coprocess stdout/err: %s' % e)
+            except Exception as error:
+                logger.warning('failed to open coprocess stdout/err: %s', error)
                 out = None
                 err = None
         else:
@@ -337,14 +349,14 @@ class Executor(object):
         try:
             proc = execute(cmd, workdir=self.__job.workdir, returnproc=True, stdout=out, stderr=err,
                            usecontainer=False, cwd=self.__job.workdir, job=self.__job)
-        except Exception as e:
-            logger.error('could not execute: %s' % str(e))
+        except Exception as error:
+            logger.error('could not execute: %s', error)
             return None
-        if type(proc) == tuple and not proc[0]:
+        if isinstance(proc, tuple) and not proc[0]:
             logger.error('failed to execute command')
             return None
 
-        logger.info('started %s -- pid=%s executable=%s' % (label, proc.pid, cmd))
+        logger.info('started %s -- pid=%s executable=%s', label, proc.pid, cmd)
 
         return proc
 
@@ -365,25 +377,23 @@ class Executor(object):
         # add time for PILOT_PRE_PAYLOAD
         self.pre_payload(job)
 
-        logger.info("\n\npayload execution command:\n\n%s\n" % cmd)
+        logger.info("\n\npayload execution command:\n\n%s\n", cmd)
         try:
             proc = execute(cmd, workdir=job.workdir, returnproc=True,
                            usecontainer=True, stdout=out, stderr=err, cwd=job.workdir, job=job)
-        except Exception as e:
-            logger.error('could not execute: %s' % str(e))
+        except Exception as error:
+            logger.error('could not execute: %s', error)
             return None
-        if type(proc) == tuple and not proc[0]:
+        if isinstance(proc, tuple) and not proc[0]:
             logger.error('failed to execute payload')
             return None
 
-        logger.info('started -- pid=%s executable=%s' % (proc.pid, cmd))
+        logger.info('started -- pid=%s executable=%s', proc.pid, cmd)
         job.pid = proc.pid
         job.pgrp = os.getpgid(job.pid)
         set_pilot_state(job=job, state="running")
 
         #_cmd = self.utility_with_payload(job)
-        #if _cmd:
-        #    logger.info('could have executed: %s' % _cmd)
 
         self.utility_after_payload_started(job)
 
@@ -398,13 +408,17 @@ class Executor(object):
         :return: updated secondary command (string).
         """
 
-        def cut_str_from(_cmd, s):
-            # cut the string from the position of the given _cmd
-            return _cmd[:_cmd.find(s)]
+        def cut_str_from(_cmd, _str):
+            """
+            Cut the string from the position of the given _cmd
+            """
+            return _cmd[:_cmd.find(_str)]
 
         def cut_str_from_last_semicolon(_cmd):
-            # cut the string from the last semicolon
-            # NOTE: this will not work if jobParams also contain ;
+            """
+            Cut the string from the last semicolon
+            NOTE: this will not work if jobParams also contain ;
+            """
             # remove any trailing spaces and ;-signs
             _cmd = _cmd.strip()
             _cmd = _cmd[:-1] if _cmd.endswith(';') else _cmd
@@ -445,16 +459,16 @@ class Executor(object):
             time.sleep(0.1)
 
             iteration += 1
-            for i in range(60):  # Python 2/3
+            for _ in range(60):  # Python 2/3
                 if args.graceful_stop.is_set():
                     breaker = True
-                    logger.info('breaking -- sending SIGTERM pid=%s' % proc.pid)
+                    logger.info('breaking -- sending SIGTERM pid=%s', proc.pid)
                     os.killpg(os.getpgid(proc.pid), signal.SIGTERM)
                     # proc.terminate()
                     break
                 time.sleep(1)
             if breaker:
-                logger.info('breaking -- sleep 3s before sending SIGKILL pid=%s' % proc.pid)
+                logger.info('breaking -- sleep 3s before sending SIGKILL pid=%s', proc.pid)
                 time.sleep(3)
                 proc.kill()
                 break
@@ -462,7 +476,7 @@ class Executor(object):
             exit_code = proc.poll()
 
             if iteration % 10 == 0:
-                logger.info('running: iteration=%d pid=%s exit_code=%s' % (iteration, proc.pid, exit_code))
+                logger.info('running: iteration=%d pid=%s exit_code=%s', iteration, proc.pid, exit_code)
             if exit_code is not None:
                 break
             else:
@@ -478,15 +492,12 @@ class Executor(object):
         :return: command (string).
         """
 
-        show_memory_usage()
-
         cmd = ""
         # for testing looping job: cmd = user.get_payload_command(job) + ';sleep 240'
         try:
             pilot_user = os.environ.get('PILOT_USER', 'generic').lower()
             user = __import__('pilot.user.%s.common' % pilot_user, globals(), locals(), [pilot_user],
                               0)  # Python 2/3
-            show_memory_usage()
             cmd = user.get_payload_command(job)  #+ 'sleep 1000'  # to test looping jobs
         except PilotException as error:
             self.post_setup(job)
@@ -495,7 +506,7 @@ class Executor(object):
             job.piloterrorcodes, job.piloterrordiags = errors.add_error_code(error.get_error_code())
             self.__traces.pilot['error_code'] = job.piloterrorcodes[0]
             logger.fatal(
-                'could not define payload command (traces error set to: %d)' % self.__traces.pilot['error_code'])
+                'could not define payload command (traces error set to: %d)', self.__traces.pilot['error_code'])
 
         return cmd
 
@@ -512,25 +523,29 @@ class Executor(object):
         try:
             # note: this might update the jobparams
             cmd_before_payload = self.utility_before_payload(job)
-        except Exception as e:
-            logger.error(e)
-            raise e
+        except Exception as error:
+            logger.error(error)
+            raise error
 
         if cmd_before_payload:
             cmd_before_payload = job.setup + cmd_before_payload
-            logger.info("\n\npreprocess execution command:\n\n%s\n" % cmd_before_payload)
+            logger.info("\n\npreprocess execution command:\n\n%s\n", cmd_before_payload)
             exit_code = self.execute_utility_command(cmd_before_payload, job, 'preprocess')
             if exit_code == 160:
-                logger.fatal('no more HP points - time to abort processing loop')
+                logger.warning('no more HP points - time to abort processing loop')
+            elif exit_code == 161:
+                logger.warning('no more HP points but at least one point was processed - time to abort processing loop')
+            elif exit_code == 162:
+                logger.warning('loop count reached the limit - time to abort processing loop')
             elif exit_code:
                 # set error code
                 job.piloterrorcodes, job.piloterrordiags = errors.add_error_code(errors.PREPROCESSFAILURE)
-                logger.fatal('cannot continue since preprocess failed: exit_code=%d' % exit_code)
+                logger.fatal('cannot continue since preprocess failed: exit_code=%d', exit_code)
             else:
                 # in case the preprocess produced a command, chmod it
                 path = os.path.join(job.workdir, job.containeroptions.get('containerExec', 'does_not_exist'))
                 if os.path.exists(path):
-                    logger.debug('chmod 0o755: %s' % path)
+                    logger.debug('chmod 0o755: %s', path)
                     os.chmod(path, 0o755)
 
         return exit_code
@@ -546,8 +561,6 @@ class Executor(object):
         # get the payload command from the user specific code
         self.pre_setup(self.__job)
 
-        show_memory_usage()
-
         cmd = self.get_payload_command(self.__job)
         # extract the setup in case the preprocess command needs it
         self.__job.setup = self.extract_setup(cmd)
@@ -557,7 +570,8 @@ class Executor(object):
         # abort when nothing more to run, or when the preprocess returns a special exit code
         iteration = 0
         while True:
-            logger.info('payload iteration loop #%d' % (iteration + 1))
+
+            logger.info('payload iteration loop #%d', iteration + 1)
             os.environ['PILOT_EXEC_ITERATION_COUNT'] = '%s' % iteration
             show_memory_usage()
 
@@ -566,7 +580,7 @@ class Executor(object):
             exit_code = self.run_preprocess(self.__job)
             jobparams_post = self.__job.jobparams
             if exit_code:
-                if exit_code == 160:
+                if exit_code >= 160 and exit_code <= 162:
                     exit_code = 0
                     # wipe the output file list since there won't be any new files
                     # any output files from previous iterations, should have been transferred already
@@ -581,6 +595,11 @@ class Executor(object):
             # now run the main payload, when it finishes, run the postprocess (if necessary)
             # note: no need to run any main payload in HPO Horovod jobs on Kubernetes
             if os.environ.get('HARVESTER_HOROVOD', '') == '':
+
+                #exit_code, _stdout, _stderr = execute('pgrep -x xrootd | awk \'{print \"ps -p \"$1\" -o args --no-headers --cols 300\"}\' | sh')
+                #logger.debug('[before payload start] stdout=%s', _stdout)
+                #logger.debug('[before payload start] stderr=%s', _stderr)
+
                 proc = self.run_payload(self.__job, cmd, self.__out, self.__err)
             else:
                 proc = None
@@ -590,7 +609,7 @@ class Executor(object):
                 # run the post-process command even if there was no main payload
                 if os.environ.get('HARVESTER_HOROVOD', '') != '':
                     logger.info('No need to execute any main payload')
-                    exit_code = self.run_utility_after_payload_finished()
+                    exit_code = self.run_utility_after_payload_finished(True, UTILITY_AFTER_PAYLOAD_FINISHED2)
                     self.post_payload(self.__job)
                 else:
                     break
@@ -608,7 +627,7 @@ class Executor(object):
                 # allow for a secondary command to be started after the payload (e.g. a coprocess)
                 utility_cmd = self.get_utility_command(order=UTILITY_AFTER_PAYLOAD_STARTED2)
                 if utility_cmd:
-                    logger.debug('starting utility command: %s' % utility_cmd)
+                    logger.debug('starting utility command: %s', utility_cmd)
                     label = 'coprocess' if 'coprocess' in utility_cmd else None
                     proc_co = self.run_command(utility_cmd, label=label)
 
@@ -625,19 +644,23 @@ class Executor(object):
                 else:
                     state = 'finished' if exit_code == 0 else 'failed'
                 set_pilot_state(job=self.__job, state=state)
-                logger.info('\n\nfinished pid=%s exit_code=%s state=%s\n' % (proc.pid, exit_code, self.__job.state))
+                logger.info('\n\nfinished pid=%s exit_code=%s state=%s\n', proc.pid, exit_code, self.__job.state)
+
+                #exit_code, _stdout, _stderr = execute('pgrep -x xrootd | awk \'{print \"ps -p \"$1\" -o args --no-headers --cols 300\"}\' | sh')
+                #logger.debug('[after payload finish] stdout=%s', _stdout)
+                #logger.debug('[after payload finish] stderr=%s', _stderr)
 
                 # stop the utility command (e.g. a coprocess if necessary
                 if proc_co:
-                    logger.debug('stopping utility command: %s' % utility_cmd)
+                    logger.debug('stopping utility command: %s', utility_cmd)
                     kill_processes(proc_co.pid)
 
                 if exit_code is None:
                     logger.warning('detected unset exit_code from wait_graceful - reset to -1')
                     exit_code = -1
 
-                if state != 'failed':
-                    exit_code = self.run_utility_after_payload_finished()
+                for order in [UTILITY_AFTER_PAYLOAD_FINISHED, UTILITY_AFTER_PAYLOAD_FINISHED2]:
+                    exit_code = self.run_utility_after_payload_finished(state, order)
 
                 self.post_payload(self.__job)
 
@@ -654,26 +677,44 @@ class Executor(object):
 
         return exit_code
 
-    def run_utility_after_payload_finished(self):
+    def run_utility_after_payload_finished(self, state, order):
         """
         Run utility command after the main payload has finished.
+        In horovod mode, select the corresponding post-process. Otherwise, select different post-process (e.g. Xcache).
 
+        The order constant can be UTILITY_AFTER_PAYLOAD_FINISHED, UTILITY_AFTER_PAYLOAD_FINISHED2
+
+        :param state: payload state; finished/failed (string).
+        :param order: constant used for utility selection (constant).
         :return: exit code (int).
         """
 
         exit_code = 0
         try:
-            cmd_after_payload = self.utility_after_payload_finished(self.__job)
-        except Exception as e:
-            logger.error(e)
+            cmd_after_payload, label = self.utility_after_payload_finished(self.__job, order)
+        except Exception as error:
+            logger.error(error)
         else:
-            if cmd_after_payload and self.__job.postprocess:
+            if cmd_after_payload and self.__job.postprocess and state != 'failed':
                 cmd_after_payload = self.__job.setup + cmd_after_payload
-                logger.info("\n\npostprocess execution command:\n\n%s\n" % cmd_after_payload)
-                exit_code = self.execute_utility_command(cmd_after_payload, self.__job, 'postprocess')
+                logger.info("\n\npostprocess execution command:\n\n%s\n", cmd_after_payload)
+                exit_code = self.execute_utility_command(cmd_after_payload, self.__job, label)
             elif cmd_after_payload:
-                logger.info("\n\npostprocess execution command:\n\n%s\n" % cmd_after_payload)
-                exit_code = self.execute_utility_command(cmd_after_payload, self.__job, 'xcache')
+                logger.info("\n\npostprocess execution command:\n\n%s\n", cmd_after_payload)
+
+                # xcache debug
+                #if 'xcache' in cmd_after_payload:
+                #    _exit_code, _stdout, _stderr = execute('pgrep -x xrootd | awk \'{print \"ps -p \"$1\" -o args --no-headers --cols 300\"}\' | sh')
+                #    logger.debug('[before xcache kill] stdout=%s', _stdout)
+                #    logger.debug('[before xcache kill] stderr=%s', _stderr)
+
+                exit_code = self.execute_utility_command(cmd_after_payload, self.__job, label)
+
+                # xcache debug
+                #if 'xcache' in cmd_after_payload:
+                #    _exit_code, _stdout, _stderr = execute('pgrep -x xrootd | awk \'{print \"ps -p \"$1\" -o args --no-headers --cols 300\"}\' | sh')
+                #    logger.debug('[after xcache kill] stdout=%s', _stdout)
+                #    logger.debug('[after xcache kill] stderr=%s', _stderr)
 
         return exit_code
 
@@ -691,11 +732,11 @@ class Executor(object):
             if utproc:
                 user = __import__('pilot.user.%s.common' % pilot_user, globals(), locals(), [pilot_user], 0)  # Python 2/3
                 sig = user.get_utility_command_kill_signal(utcmd)
-                logger.info("stopping process \'%s\' with signal %d" % (utcmd, sig))
+                logger.info("stopping process \'%s\' with signal %d", utcmd, sig)
                 try:
                     os.killpg(os.getpgid(utproc.pid), sig)
-                except Exception as e:
-                    logger.warning('exception caught: %s (ignoring)' % e)
+                except Exception as error:
+                    logger.warning('exception caught: %s (ignoring)', error)
 
                 user.post_utility_command_action(utcmd, self.__job)
 
@@ -712,4 +753,4 @@ class Executor(object):
             if os.path.exists(name):
                 os.rename(name, name + '%d' % iteration)
             else:
-                logger.warning('cannot rename %s since it does not exist' % name)
+                logger.warning('cannot rename %s since it does not exist', name)
