@@ -5,13 +5,13 @@
 # http://www.apache.org/licenses/LICENSE-2.0
 #
 # Authors:
-# - Paul Nilsson, paul.nilsson@cern.ch, 2018-2020
+# - Paul Nilsson, paul.nilsson@cern.ch, 2018-2021
 
 from pilot.common.errorcodes import ErrorCodes
-from pilot.util.auxiliary import whoami, set_pilot_state, cut_output
+from pilot.util.auxiliary import whoami, set_pilot_state, cut_output, locate_core_file
 from pilot.util.config import config
 from pilot.util.container import execute
-from pilot.util.filehandling import remove_files, find_latest_modified_file, verify_file_list
+from pilot.util.filehandling import remove_files, find_latest_modified_file, verify_file_list, copy
 from pilot.util.parameters import convert_to_int
 from pilot.util.processes import kill_processes
 from pilot.util.timing import time_stamp
@@ -59,20 +59,49 @@ def looping_job(job, mt):
         # the payload process is considered to be looping if it's files have not been touched within looping_limit time
         if time_last_touched:
             ct = int(time.time())
-            logger.info('current time: %d' % ct)
-            logger.info('last time files were touched: %d' % time_last_touched)
-            logger.info('looping limit: %d s' % looping_limit)
+            logger.info('current time: %d', ct)
+            logger.info('last time files were touched: %d', time_last_touched)
+            logger.info('looping limit: %d s', looping_limit)
             if ct - time_last_touched > looping_limit:
                 try:
+                    # first produce core dump and copy it
+                    create_core_dump(pid=job.pid, workdir=job.workdir)
+                    # set debug mode to prevent core file from being removed before log creation
+                    job.debug = True
                     kill_looping_job(job)
-                except Exception as e:
-                    logger.warning('exception caught: %s' % e)
+                except Exception as error:
+                    logger.warning('exception caught: %s', error)
         else:
             logger.info('no files were touched')
 
     return exit_code, diagnostics
 
 
+def create_core_dump(pid=None, workdir=None):
+    """
+    Create core dump and copy it to work directory
+    """
+
+    if not pid or not workdir:
+        logger.warning('cannot create core file since pid or workdir is unknown')
+        return
+
+    cmd = 'gdb --pid %d -ex \'generate-core-file\'' % pid
+    exit_code, stdout, stderr = execute(cmd)
+    if not exit_code:
+        path = locate_core_file(pid=pid)
+        if path:
+            try:
+                copy(path, workdir)
+            except Exception as error:
+                logger.warning('failed to copy core file: %s', error)
+            else:
+                logger.debug('copied core dump to workdir')
+
+    else:
+        logger.warning('failed to execute command: %s, stdout+err=%s', cmd, stdout + stderr)
+
+
 def get_time_for_last_touch(job, mt, looping_limit):
     """
     Return the time when the files in the workdir were last touched.
@@ -98,14 +127,14 @@ def get_time_for_last_touch(job, mt, looping_limit):
             # remove unwanted list items (*.py, *.pyc, workdir, ...)
             files = loopingjob_definitions.remove_unwanted_files(job.workdir, files)
             if files:
-                logger.info('found %d files that were recently updated' % len(files))
-                logger.debug('recent files:\n%s' % files)
+                logger.info('found %d files that were recently updated', len(files))
+                logger.debug('recent files:\n%s', files)
                 updated_files = verify_file_list(files)
 
                 # now get the mod times for these file, and identify the most recently update file
                 latest_modified_file, mtime = find_latest_modified_file(updated_files)
                 if latest_modified_file:
-                    logger.info("file %s is the most recently updated file (at time=%d)" % (latest_modified_file, mtime))
+                    logger.info("file %s is the most recently updated file (at time=%d)", latest_modified_file, mtime)
                 else:
                     logger.warning('looping job algorithm failed to identify latest updated file')
                     return mt.ct_looping_last_touched
@@ -120,7 +149,7 @@ def get_time_for_last_touch(job, mt, looping_limit):
         # cut the output if too long
         stdout = cut_output(stdout)
         stderr = cut_output(stderr)
-        logger.warning('find command failed: %d, %s, %s' % (exit_code, stdout, stderr))
+        logger.warning('find command failed: %d, %s, %s', exit_code, stdout, stderr)
 
     return mt.ct_looping_last_touched
 
@@ -140,19 +169,19 @@ def kill_looping_job(job):
 
     cmd = 'ps -fwu %s' % whoami()
     exit_code, stdout, stderr = execute(cmd, mute=True)
-    logger.info("%s: %s" % (cmd + '\n', stdout))
+    logger.info("%s: %s", cmd + '\n', stdout)
 
     cmd = 'ls -ltr %s' % (job.workdir)
     exit_code, stdout, stderr = execute(cmd, mute=True)
-    logger.info("%s: %s" % (cmd + '\n', stdout))
+    logger.info("%s: %s", cmd + '\n', stdout)
 
     cmd = 'ps -o pid,ppid,sid,pgid,tpgid,stat,comm -u %s' % whoami()
     exit_code, stdout, stderr = execute(cmd, mute=True)
-    logger.info("%s: %s" % (cmd + '\n', stdout))
+    logger.info("%s: %s", cmd + '\n', stdout)
 
     cmd = 'pstree -g -a'
     exit_code, stdout, stderr = execute(cmd, mute=True)
-    logger.info("%s: %s" % (cmd + '\n', stdout))
+    logger.info("%s: %s", cmd + '\n', stdout)
 
     # set the relevant error code
     if job.state == 'stagein':
@@ -184,6 +213,6 @@ def get_looping_job_limit():
     looping_limit = convert_to_int(config.Pilot.looping_limit_default, default=2 * 3600)
     looping_limit_min_default = convert_to_int(config.Pilot.looping_limit_min_default, default=2 * 3600)
     looping_limit = max(looping_limit, looping_limit_min_default)
-    logger.info("using looping job limit: %d s" % looping_limit)
+    logger.info("using looping job limit: %d s", looping_limit)
 
     return looping_limit
